//PCF programming in Java



package com.dmitri.pcf;

import com.ibm.mq.pcf.*;
import com.ibm.mq.*;
import java.util.*;
/**
*  @author Dmitri
*
*   This class extends Hashtable to provide PCF-specific processing.
*/
public class PCFHashtable extends Hashtable {

  private int iReasonCode;

/**
*   Constructor that sets up the data based on a message.
*/
  PCFHashtable(MQMessage message) {

    super();
    try {
      MQCFH cfh = new MQCFH(message);
      iReasonCode = cfh.reason;

      if (isValid()) {

        PCFParameter p;
        for (int i = 0; i < cfh.parameterCount; i++) {

          // Walk through the returned attributes
          p = PCFParameter.nextParameter (message);
          Integer key = new Integer(p.getParameter());
          put(key, p.getValue());
        }
      }
    } catch (Exception ex) {
      System.out.println(ex);
    }
  }

/**
*   Is the PCF request valid and, hence, may it be used?
*/
  public boolean isValid() {
    return iReasonCode == 0;
  }

/**
*   Returns the reason code for use by diagnostic processes.
*/
  public int getReasonCode(){
    return iReasonCode;
  }

/**
*   Returns the int value represented by the key supplied.
*/
  public int getIntValue(int key) {
    Integer i = (Integer) get(new Integer(key));
    return i.intValue();
  }
/**
*   Returns the int array represented by the key supplied.
*/
  public int [] getIntArray(int key) {
    return (int []) get(new Integer(key));
  }
/**
*   Returns the string value represented by the key supplied.
*/
  public String getStringValue(int key) {
    // mq returns padded strings.
    return ((String) get(new Integer(key))).trim();
  }
/**
*   Returns the string array represented by the key supplied.
*/
  public String [] getStringArray(int key) {
    String [] paddedStrings =  (String []) get(new Integer(key));
    String [] trimStrings = new String [paddedStrings.length];
    for (int i = 0; i < paddedStrings.length; i++) {
      trimStrings[i] = paddedStrings[i].trim();
    }
    return trimStrings;
  }
}


package com.dmitri.pcf;
import com.ibm.mq.pcf.*;
import com.ibm.mq.*;

/**
*   @author Dmitri
*
*   This class is used to query queue managers for details of all
*   the channels they own.
*   This query is currently limited to sender channels.
*/
public class ChannelDetailsPCF implements CMQCFC {

  private PCFAgent iAgent;
  private PCFParameter []  iParameters;

/**
*   This constructor requires a valid queue manager agent. This 
*   "channel details" request is currently only for sender channels
*   (channel type of MQCHT_SENDER), though this may easily be
*   changed to MQCHT_ALL.
*/
  public ChannelDetailsPCF(PCFAgent agent) {

    iAgent = agent;
    iParameters = new PCFParameter [] {
        new MQCFST (MQCACH_CHANNEL_NAME, "*"),
//      new MQCFIN (MQIACH_CHANNEL_TYPE, CMQXC.MQCHT_SENDER),
        new MQCFIN (MQIACH_CHANNEL_TYPE, CMQXC.MQCHT_ALL),
        new MQCFIL (MQIACF_CHANNEL_ATTRS, new int [] {MQIACF_ALL} ),
        };
  }

/**
*  This method returns an array of PCFHashtables. Each hashtable
*  represents a single channel. The hashtable holds all data
*  pertaining to the channel, and the data varies according to the
*  channel, so that, for example, the sender channel holds details
*  of target queue manager, while the receiver channel doesn't.
*/
  public PCFHashtable [] getChannels() {

    try {
      MQMessage [] pcfResponses = iAgent.send (MQCMD_INQUIRE_CHANNEL, 
                                                          iParameters);
      PCFHashtable [] details = new PCFHashtable [pcfResponses.length];

      for (int i = 0; i < pcfResponses.length; i++) {
        details[i] = new PCFHashtable(pcfResponses[i]);
      }
      return details;

    } catch (Exception e) {
      e.printStackTrace();
    }
    return null;
  }
}


package com.dmitri.pcf;
import com.ibm.mq.pcf.*;
import com.ibm.mq.*;
/**
*   @author Dmitri
*
*   This class is used to query channel status.
*/
public class ChannelStatusPCF implements CMQCFC {

  private PCFAgent iAgent;
  private PCFParameter []  iParameters;

/**
*   Constructor (requires a valid queue manager agent).
*/
  public ChannelStatusPCF(PCFAgent agent) {

    iAgent = agent;
    iParameters =   new PCFParameter [] {
            new MQCFST (MQCACH_CHANNEL_NAME, "*"),
            new MQCFIL (MQIACH_CHANNEL_INSTANCE_ATTRS, new int [] 
                                                       {MQIACF_ALL} ),
            };
  }

/**
*  This method returns an array of PCFHashtables. Each hashtable
*  contains parameters describing channel status.
*/
  public PCFHashtable [] getChannelStatus() {

    try {
      MQMessage [] pcfResponses = iAgent.send 
                           (MQCMD_INQUIRE_CHANNEL_STATUS, iParameters);
      PCFHashtable [] details = new PCFHashtable [pcfResponses.length];

      for (int i = 0; i < pcfResponses.length; i++) {
        details[i] = new PCFHashtable(pcfResponses[i]);
      }
      return details;

    } catch (Exception e) {
      e.printStackTrace();
    }
  return null;
  }
}


package com.dmitri.pcf;

import com.ibm.mq.*;
import java.util.*;
/**
*   @author Dmitri
*
*  This class is the top-level module for the MQ Infrastructure
*  monitor suite. It is a coordination and look-up point for other
*  classes, and makes use of a standard Singleton pattern to ensure
*  that it is not duplicated.
*/
public class MQInfrastructure {

  private static MQInfrastructure cInstance;
  private Hashtable iAllQueueManagers = new Hashtable();

/**
*  Singleton private constructor. This sets up the first queue manager
*  (specified by a command line argument), then calls the method
*  discoverAllQmgrs to crawl through the MQSeries infrastructure.
*/
  private MQInfrastructure(String startQmgrName, int startPort, 
                                                String startAddress) {
    QMgrManager mgr = new QMgrManager(startQmgrName, startPort, 
                                                startAddress);
    try {
      mgr.connect();
      mgr.queryChannels();

      iAllQueueManagers.put(mgr.getName(), mgr);
      mgr.discoverQmgrs(iAllQueueManagers);
      discoverAllQmgrs(iAllQueueManagers);

    } catch (MQException ex) {

      ex.printStackTrace();
    }
  }

/**
*   Provides access to the singleton instance.
*/
  public static MQInfrastructure instance() {
    return cInstance;
  }

/**
*   Provides access to the queue manager data.
*/
  public Hashtable getQueueManagers() {
    return iAllQueueManagers;
  }

/**
*  Iterate through the listed queue managers, connecting and querying
*  until no more are discovered. This could be described as a "crawler
*  method", as it uses a queue manager's references to other queue
*  managers for "auto discovery" (as long as conventions are followed
*  and queue managers are running).
*
*  I've opted for an interative rather than a recursive approach here
*  as it makes everything a bit more explicit.
*/
  private void discoverAllQmgrs(Hashtable qmgrStore) {

    int currentSize = qmgrStore.size();
    for (int i = 0; i != currentSize; ) {

      i = currentSize;

      for (Enumeration enum = qmgrStore.elements();
                                          enum.hasMoreElements(); ) {

        QMgrManager current = (QMgrManager) enum.nextElement();
        if (!current.isConnected()) {
        // This is an area that needs to be improved. If a queue 
        // manager refuses a connection for whatever reason, this
        // code attempts to re-connect on each sweep. While this
        // may be desirable, it imposes a processing overhead.
          try {
            current.connect();
            current.queryChannels();
            current.discoverQmgrs(qmgrStore);
          } catch (MQException ex) {
            System.out.println(ex);
          }
        }
      }
      currentSize = qmgrStore.size();
    }
  }

/**
*  Bootstrap method. Requires a starting point (queue manager name
*  and the port and address on which it's running.
*/
  public static void main(String [] args) {

    if (args.length != 3) {
      remindOfUsage();
    }
    try {
      int port = Integer.parseInt( args[1] );
      cInstance = new MQInfrastructure( args[0], port, args[2]);
      printAllQueueManagerDetails();

    } catch (NumberFormatException nex) {
      remindOfUsage();
    }
  }

/**
*   A gentle reminder of how to use this program.
*/
  private static void remindOfUsage() {

    System.out.println("Usage:");
    System.out.println("java com.dmitri.pcf.MQInfrastructure 
                                       QueueManagerName Port Address");
    System.out.println("Note: MQSeries will use port 1414 as default");

    System.exit(0);
  }

/**
*  This is a very simple way of rendering all the data that is 
*  collected. It iterates through all queue manager data and dumps
*  it to system out.
*/
  private static void printAllQueueManagerDetails() {

    for (Enumeration enum = instance().getQueueManagers().elements(); 
                                          enum.hasMoreElements(); ) {

      QMgrManager current = (QMgrManager) enum.nextElement();
      System.out.println("---------------------------------------------------------");
      System.out.println("Queue manager : " + current.getName());
      System.out.println("");
      System.out.println("Has channels:");

      ManagedChannel [] chls = current.getChannels();
      for (int i = 0; i < chls.length; i++) {

        System.out.println( chls[i].getName() + " : " + 
                                        chls[i].getChannelStatus() );
      }
    }
  }
}


package com.dmitri.pcf;

import com.ibm.mq.pcf.*;
import com.ibm.mq.*;
import java.util.*;
/**
*  @author Dmitri
*
*  This class is used to control all access to managed MQSeries
*  queue managers. It controls access to resources.
*
*  This class relies both on queue managers being accessible via
*  client connections (TCP/IP) and the command server being running.
*/
public class QMgrManager implements CMQCFC {

  private String iHostname;
  private int iPort;
  private String iQmgrName;
  private String iClientChannel;
  private PCFAgent iAgent;
  private boolean iIsConnected = false;

  private ChannelDetailsPCF iDetailsPCF;
  private ChannelStatusPCF iStatusPCF;

  private Hashtable iManagedChannels = new Hashtable();

  public final static String CLIENT_CHANNEL = "SYSTEM.DEF.SVRCONN";

/**
*  The constructor (it assumes the default CLIENT_CHANNEL is used
*  to reach the destination qmgr).
*/
  public QMgrManager(String qmgrName, int port, String hostname) {
    this(qmgrName, hostname, port, CLIENT_CHANNEL);
  }

/**
*   The full constructor.
*/
  public QMgrManager(String qmgrName, String hostname, int port,
                                            String clientChannel) {

    iQmgrName = qmgrName;
    iHostname = hostname;
    iPort = port;
    iClientChannel = clientChannel;
  }

/**
*  Connects to the queue manager so that it can be queried. Also 
*  prepares PCF queries.
*/
  public void connect() throws MQException {

    iAgent = new PCFAgent(iHostname, iPort, iClientChannel);
    iDetailsPCF = new ChannelDetailsPCF(iAgent);
    iStatusPCF = new ChannelStatusPCF(iAgent);
    iIsConnected = true;
  }

/**
*  Used to determine whether this is an active queue manager.
*/
  public boolean isConnected() {
    return iIsConnected;
  }

/**
*  Returns the queue manager's name.
*/
  public String getName() {
    return iQmgrName;
  }

/**
*  Query the queue manager for channel details.
*/
  public void queryChannels() {

    PCFHashtable [] details = iDetailsPCF.getChannels();
    PCFHashtable [] status = iStatusPCF.getChannelStatus();
    PCFHashtable current;
    String name;

    for (int i = 0; i < details.length; i++) {
      current = null;
      name = details[i].getStringValue(MQCACH_CHANNEL_NAME);

      for (int j = 0; j < status.length; j++) {
        // try to match status with details.
        if (name.equals(status[j].getStringValue(
                                             MQCACH_CHANNEL_NAME))) {
          current = status[j];
          break;
        }
      }
      ManagedChannel mchl = (ManagedChannel) 
                                iManagedChannels.get(name);

      //instantiation is requred the first time channel is found.
      if (mchl == null) {
        mchl = new ManagedChannel(iAgent);
        iManagedChannels.put(name, mchl);
      }
      mchl.refreshChannel(details[i], current);

    }
  }

/**
*  This method may be called by a control object to query queue
*  managers for details of other queue managers, thereby discovering
*  ones not known to the control object. This is achieved by
*  delegating the call to the managed channels and querying them
*  for remote queue manager details. The hashtable returned is
*  keyed by queue manager name, and the item is a QMgrManager
*  object (not connected).
*/
  public void discoverQmgrs(Hashtable qmgrs) {

    ManagedChannel currentChl;

    for (Enumeration chls = iManagedChannels.elements();
                                         chls.hasMoreElements(); ) {

      currentChl = (ManagedChannel) chls.nextElement();
      currentChl.copyQmgrsInto(qmgrs);
    }
  }

/**
*   Returns an array of managed channels.
*/
  public ManagedChannel [] getChannels() {

    ManagedChannel [] channels = new ManagedChannel 
                                           [iManagedChannels.size()];
    int i = 0;
    for (Enumeration chls = iManagedChannels.elements(); 
                                      chls.hasMoreElements(); i++) {

      channels[i] = (ManagedChannel) chls.nextElement();
    }
        return channels;

  }
}


package com.dmitri.pcf;

import com.ibm.mq.pcf.*;
import com.ibm.mq.*;
import java.util.*;
/**
*  @author Dmitri
*
*  This class is used to control access to channels. It deals with a
*  single channel and coordinates other facets of channel management.
*
*  This is the main class that would be extended to provide a more
*  complete system management solution. For example, from here it
*  would be possible to add a channel renderer to represent a channel
*  visually. It would also be appropriate to tag on classes to alter
*  channels (eg by starting or resetting them).
*/
public class ManagedChannel implements CMQCFC, CMQXC {

  private PCFHashtable iChannelDetails;
  private PCFHashtable iChannelStatus;
  private String iName;
  private String iXmitQName;
  private PCFAgent iAgent;

  private QMgrManager iTargetQMgr;

  public static int DEFAULT_PORT = 1414;

  // lookup point to translate channel status into English.
  public static String [] CHANNEL_STATUS = {
    "Inactive",
    "Binding",
    "Starting",
    "Running",
    "Stopping",
    "Retrying",
    "Stopped",
    "Requesting",
    "Paused",
    "",
    "",
    "",
    "",
    "Initializing",
  };

/**
*  Simple constructor that saves a reference to the queue manager's
*  agent.
*/
  public ManagedChannel(PCFAgent agent) {
    super();
    iAgent = agent;
  }

/**
*  This method is called by the QMgrManager class. This is done
*  because calls to enquire channel details and status may return
*  multiple channels.
*/
  public void refreshChannel(PCFHashtable details, 
                                              PCFHashtable status) {

    iChannelDetails = details;
    iChannelStatus = status; // often null.

    iName = iChannelDetails.getStringValue(MQCACH_CHANNEL_NAME);
    refreshXmitQ();
    refreshTargetQmgr();
  }

/**
*   Return the name of the managed channel.
*/
  public String getName() {
    return iName;
  }

/**
*   Return the channel's status (string format).
*/
  public String getChannelStatus() {
    return CHANNEL_STATUS[ getChannelStatusInt() ];
  }

/**
*   Return the channel's status (int format).
*/
  public int getChannelStatusInt() {

    try {
      return iChannelStatus.getIntValue(MQIACH_CHANNEL_STATUS);

    } catch (NullPointerException channelIsInactive) {
      return 0;
    }

  }

/**
*  A possible future extension could be to get the depth of the XmitQ.
*  An XmitQ with messages on is a good sign of problems in a system
*  that requires channels to be running at all times.
*/
  private void refreshXmitQ() {

    try {
      iXmitQName = iChannelDetails.getStringValue(MQCACH_XMIT_Q_NAME);
      } catch (NullPointerException noXmitq) {
    }
  }

/**
*  This is used to discover other queue managers on a network. It
*  does this by querying sender/server channels for details of
*  partner queue managers, and makes a number of assumptions:
*
*  1  That the connection is based on TCP/IP (the whole Java MQ
*     interface relies on IP).
*  2  That the convention is followed whereby XMitQs have the same
*     name as queue managers (MQSeries management becomes difficult
*     if this convention is not followed).
*  3  That remote queue managers are accessible via the default
*     client connection channel.
*/
  private void refreshTargetQmgr() {

    if (iXmitQName == null) return;

    // If already set up...
    if (iTargetQMgr != null && 
                           TargetQMgr.getName().equals(iXmitQName)) {
      return;
    }
    int chltype = iChannelDetails.getIntValue(MQIACH_CHANNEL_TYPE);
    if (chltype != MQCHT_SENDER &&
                chltype != MQCHT_SERVER) { // we are unable to query
                                           // for connection name.
      return;
    }
    String conname = 
              iChannelDetails.getStringValue(MQCACH_CONNECTION_NAME);
    int port = DEFAULT_PORT;

    if (conname.indexOf('(') > 0) {   // port number is embedded
      port = Integer.parseInt(   conname.substring( 
                  conname.indexOf('(') + 1, conname.indexOf(')') ) );
      conname = conname.substring( 0, conname.indexOf('(') );
    }

    iTargetQMgr = new QMgrManager( iXmitQName, port, conname );

  }

/**
*  A method of accumulating all data about referenced queue
*  managers in a common place.
*/
  public void copyQmgrsInto(Hashtable qmgrs) {

    if (iTargetQMgr == null ||
      iTargetQMgr.getName().equals("") ||
      qmgrs.containsKey(iTargetQMgr.getName())) {
      return;
    }
    qmgrs.put(iTargetQMgr.getName(), iTargetQMgr);
  }
}


package com.dmitri.pcf;
import com.ibm.mq.pcf.*;
import com.ibm.mq.*;

/**
*   @author Dmitri
*
*  This class is used to start channels. It is not linked to other
*  classes and is included only as an example.
*/
public class StartChannelPCF implements CMQCFC {

  private PCFAgent iAgent;
  private PCFParameter []  iParameters;

/**
*  The constructor, which requires a valid queue manager agent.
*/
  public StartChannelPCF(PCFAgent agent) {

    iAgent = agent;
  }

/**
*  This method starts the channel specified.
*/
  public void startChannel(String channelName) {

    iParameters =    new PCFParameter [] {
            new MQCFST (MQCACH_CHANNEL_NAME, channelName),
            };

    try {
      MQMessage [] pcfResponses = iAgent.send (MQCMD_START_CHANNEL, 
                                                        iParameters);

      // Assume only the one response.
      PCFHashtable response = new PCFHashtable(pcfResponses[0]); 

      if (response.isValid()) {
        System.out.println("Channel " + channelName + " Started");

      } else {
        System.out.println( "Channel " + channelName + 
                                " failed to start, reason code : " + 
                                response.getReasonCode() );
      }

    } catch (Exception e) {
      e.printStackTrace();
    }
  }
}
